import waba.sys.*;  // Convert, Time
// StringUtil

/**
 * @author <A HREF="mailto:weyer@kagi.com">Steve Weyer</A>
 * @version 11/23/00
*/

class TimeUtil {

  private final static String[] monthNames = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
  };

  private final static int[] monthDays = {
    31, 28, 31, 30, 31, 30,
    31, 31, 30, 31, 30, 31,
  };

  // use format: dd mmm yyyy hh:mm:ss AM
  private static final char[] delims = {' ', ' ', ' ', ':', ':', ' '}; // see toString, fromString

  private static int monthNumber(String mon) { // 1-12 or -1
    for (int i = 0; i < 12; i++)
      if (monthNames[i].equals(mon)) return i+1;
    return -1;
  }

  private static boolean isLeapYear(int yr) {
    return
      (yr % 4) == 0 &&
      ((yr % 400) == 0 || (yr % 100) != 0);
  }

  private static int yearDays(int yr) {
    return (isLeapYear(yr)) ? 366 : 365;
  }

  private static int compareTo(Time t1, Time t2) { // <(-1), =(0), >(1)
    int[] fields1 = {t1.year, t1.month, t1.day, t1.hour, t1.minute, t1.second};
    int[] fields2 = {t2.year, t2.month, t2.day, t2.hour, t2.minute, t2.second};
    int field1, field2;
    for (int i=0; i < 6; i++) {
      field1 = fields1[i];
      field2 = fields2[i];
      if (field1 < field2)
        return -1;
      else if (field1 > field2)
        return 1;
    };
    return 0;
  }

  private static int yearDay(Time t) { // day # within year
    int tot = t.day, mon = t.month, yr = t.year;
    for (int m = 1; m < mon; m++)
      tot += daysInMonth(yr, m);
    return tot;
  }

  // PUBLIC

  public static String toString(Time tt) {
    int hrs = tt.hour, mins = tt.minute, secs = tt.second;
    return
      String.valueOf(tt.day) + delims[0] +
      monthNames[(tt.month < 0) ? 12 : tt.month-1] + delims[1] +
      tt.year + delims[2] +
      ((hrs <= 12) ? ((hrs == 0) ? 12 : hrs) : hrs-12) + delims[3] +
      ((mins < 10) ? "0" : "") + mins + delims[4] +
      ((secs < 10) ? "0" : "") + secs + delims[5] +
      ((hrs > 11) ? "PM" : "AM")
      ;
  }

  public static int daysInMonth(int yr, int mon) {
    return (mon == 2 && isLeapYear(yr)) ? 29 : monthDays[mon-1];
  }

  public static int dayDiff(Time t1, Time t2) {
    int comp = compareTo(t1, t2);
    if (comp > 0)
      return -dayDiff(t2, t1);
    else if (comp == 0)
      return 0;

    // t1 < t2
    int y1 = t1.year;
    int d1 = yearDay(t1);
    int y2 = t2.year;
    int d2 = yearDay(t2);
    int days;
    if (y1 == y2)
      days = d2 - d1;
    else {
      days = yearDays(y1) - d1; // remainder
      for (int y = y1+1; y < y2; y++)
        days += yearDays(y);
      days += d2;
    };

    int secs = (t2.hour - t1.hour) * 3600;
    secs += (t2.minute - t1.minute) * 60;
    secs += t2.second - t1.second;
    return (secs < 0) ? days-1 : days;
  }

  public static int secDiff(Time t1, Time t2) {
    int comp = compareTo(t1, t2);
    if (comp > 0)
      return 86400-secDiff(t2, t1);
    else if (comp == 0)
      return 0;

    // t1 < t2
    int secs = (t2.hour - t1.hour) * 3600;
    secs += (t2.minute - t1.minute) * 60;
    secs += t2.second - t1.second;
    return (secs < 0) ? 86400+secs : secs;
  }

  public static Time fromString(String tstr) {
    if (tstr.length() == 0)
      return null;
    int i, pos1 = 0, pos2 = 0, val, tlen = tstr.length();
    int[] fields = {0,0,0,0,0,0};
    String field;
    for (i = 0; i < fields.length; i++) {
      pos2 = StringUtil.indexOf(tstr, delims[i], pos1);
      if (pos2 < 0) pos2 = tlen;
      if (pos2 <= pos1) break;
      //if (DEBUG) System.out.println("i: " + i + "; pos2: " + pos2);
      field = tstr.substring(pos1,pos2);
      if (i == 1) {
        val = monthNumber(field);
        if (val < 0)  // !!!b
          return null;
      }
      else val = Convert.toInt(field);
      fields[i] = val;
      pos1 = pos2+1;
    };
    field = (pos2 == tlen) ? "AM" : tstr.substring(pos2+1, tlen); // AM/PM

    Time tt = new Time();
    tt.day = fields[0];
    tt.month = fields[1];
    tt.year = fields[2];
    int hrs = fields[3];
    tt.hour = (field.equals("AM")) ? ((hrs == 12) ? 0 : hrs) : hrs+12;
    tt.minute = fields[4];
    tt.second = fields[5];
    return tt;
  }

}